#!/usr/bin/env ruby
# Id: image_resources.rb,v 1.1 2011/11/29 15:43:26 rjs Exp 
#
# Resource-based image extractor based on PDFlib TET
#

require './TET'


# global option list
globaloptlist = "searchpath={{../data}}"

# document-specific option list
docoptlist = ""

# page-specific option list
pageoptlist = ""

# here you can insert basic image extract options (more below)
baseimageoptlist = ""

# set this to 1 to generate image data in memory

begin
    if (ARGV.length != 1)
	raise("usage: image_resource.pl <filename>\n")
    end

    tet = TET.new

    outfilebase = ARGV[0]

    tet.set_option(globaloptlist)

    doc = tet.open_document(ARGV[0], docoptlist)

    if (doc == -1)
	raise "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname() \
	    + "(): " + tet.get_errmsg()
    end

    # get number of pages in the document
    n_pages = tet.pcos_get_number(doc, "length:pages")

    # loop over pages in the document
    1.upto(n_pages) do |pageno|
	page = tet.open_page(doc, pageno, pageoptlist)

	if (page == -1)
	    print "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname()\
		+ "(): " + tet.get_errmsg()
	    next                        # try next page
	end

	if (tet.get_errnum() != 0)
	    raise "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname()\
		+ "(): " + tet.get_errmsg()
	end

	tet.close_page(page)
    end

    # get number of image resources in the document
    n_images = tet.pcos_get_number(doc, "length:images")

    # loop over image resources in the document
    0.upto(n_images-1) do |imageid|
	# examine image type 
	mergetype = tet.pcos_get_number(doc, \
			    "images[" + imageid.to_s + "]/mergetype")

	# skip images which have been consumed by merging
	if (mergetype == 0 || mergetype == 1)
	    # Print the following information for each image:
	    # - image number
	    # - pCOS id (required for indexing the images[] array)
	    # - physical size of the placed image on the page
	    # - pixel size of the underlying PDF image
	    # - number of components, bits per component, and colorspace
	    # - mergetype if different from "normal", i.e. "artificial"
	    #   (=merged) or "consumed"
	    width = tet.pcos_get_number(doc, "images[" + imageid.to_s + "]/Width")
	    height = tet.pcos_get_number(doc, "images[" + imageid.to_s + "]/Height")
	    bpc = tet.pcos_get_number(doc, "images[" + imageid.to_s + "]/bpc")
	    cs = tet.pcos_get_number(doc, "images[" + imageid.to_s + "]/colorspaceid")

	    printf("image I%d: %dx%d pixel, ", imageid, width, height)


	    if (cs != -1)
		printf("%dx%d bit %s", \
		    tet.pcos_get_number(doc, \
			    "colorspaces[" + (cs.to_i).to_s + "]/components"), bpc, \
		    tet.pcos_get_string(doc, "colorspaces[" + (cs.to_i).to_s + "]/name"))
	    else
		# cs==-1 may happen for some JPEG 2000 images. bpc,
		# colorspace name and number of components are not
		# available in this case.
		printf("JPEG2000")
	    end

	    if (mergetype != 0)
		printf(", mergetype=")

		if (mergetype == 1)
		    printf("artificial")
		else
		    printf("consumed")
		end
	    end
	    printf("\n")

	    #
	    # Fetch the image data and write it to a disk file. The
	    # output filenames are generated from the input filename
	    # by appending page number and image number.
	    imageoptlist = sprintf("%s filename={%s_I%d}",
		baseimageoptlist, outfilebase, imageid)
	    if (tet.write_image_file(doc, imageid, imageoptlist) == -1)
		raise "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname() \
		    + "(): " + tet.get_errmsg()
		next                  # process next image
	    end
	end
    end

    tet.close_document(doc)

rescue TETException => pe
    print pe.backtrace.join("\n")
    printf("TET exception occurred in image_resources:\n")
    print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
                        ": " + pe.get_errmsg + "\n"
rescue Exception => e
    print e.backtrace.join("\n") + "\n" + e.to_s + "\n"
ensure
    tet.delete() if tet
end
